﻿/*

  Pdf viewer

  Source files:
    PdfView.prg
    PdfView_C.prg

  Compiler: HMG 3.4.4
  Link with library: SumatraPDF

*/


#include "Directry.ch"
#include "hmg.ch"
#include "PdfView.ch"

STATIC lPdfView_Max
STATIC nPdfView_R
STATIC nPdfView_C
STATIC nPdfView_W
STATIC nPdfView_H
STATIC nFiles_W
STATIC nZoom
STATIC lMenuBar
STATIC lToolbar
STATIC lBookmarks
STATIC lOpenAtOnce
STATIC lEscExit
STATIC cSumatraDir
STATIC cFileDir
STATIC cFileLast
STATIC cLang

STATIC cPanel
STATIC lMenuActive
STATIC nHFocusMenu
STATIC nHMenuMain

MEMVAR _HMG_SYSDATA
MEMVAR _HMG_DEFAULTICONNAME


FUNCTION Main()

  SET FONT TO "MS Shell Dlg", 8
  SET DEFAULT ICON TO "SUMATRA"

  SettingsRead()

  IF IsProgramRunning()
    QUIT
  ENDIF

  cPanel      := "SumatraPanel"
  lMenuActive := .F.

  DEFINE WINDOW PdfView;
    ROW    nPdfView_R;
    COL    nPdfView_C;
    WIDTH  nPdfView_W;
    HEIGHT nPdfView_H;
    TITLE  "PdfView";
    MAIN;
    ON INIT      FileListRefresh(cFileLast);
    ON GOTFOCUS  PdfView.PdfTimer.ENABLED := .T.;
    ON LOSTFOCUS PdfView.PdfTimer.ENABLED := .F.;
    ON SIZE      ResizeControls();
    ON MAXIMIZE  ResizeControls();
    ON RELEASE   (SettingsWrite(), Sumatra_FileClose(cPanel), DestroyMenu(nHMenuMain))

    DEFINE GRID Files
      ROW            0
      COL            0
      WIDTH          nFiles_W
      HEADERS        {NIL, NIL, NIL, NIL, NIL}
      WIDTHS         {nFiles_W - 4, 0, 0, 0, 0}
      CELLNAVIGATION .F.
      ONDBLCLICK     FileOpen()
      ONCHANGE       If(lOpenAtOnce .and. (! ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR))), FileOpen(), NIL)
      ONKEY          GridOnKey()
    END GRID

    DEFINE WINDOW &(cPanel);
      PANEL;
      ON PAINT Sumatra_FrameRedraw(cPanel)
    END WINDOW

    DEFINE STATUSBAR
      STATUSITEM ""
      STATUSITEM "" WIDTH 140
    END STATUSBAR

    DEFINE TIMER PdfTimer INTERVAL 100 ACTION SetStatusPage(.F.)
  END WINDOW

  ListView_ChangeExtendedStyle(PdfView.Files.HANDLE, LVS_EX_INFOTIP)
  HMG_ChangeWindowStyle(GetFormHandle(cPanel), WS_SIZEBOX, NIL, .F., .T.)

  IF lPdfView_Max
    PdfView.MAXIMIZE
  ENDIF

  SetOnKey(.T.)
  SetLangInterface()
  ResizeControls()
  InstallEventHandler("EventHandler")

  PdfView.ACTIVATE

RETURN NIL


FUNCTION EventHandler(nHWnd, nMsg, nWParam, nLParam)
  LOCAL nMaxY
  LOCAL aRect1, aRect2
  LOCAL nCol

  IF nHWnd == PdfView.HANDLE
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        SetMinMaxTrackSize(nLParam, 400, 300)
        EXIT

      CASE 274 /*WM_SYSCOMMAND*/
        IF HB_BitAnd(nWParam, 0xFFF0) == 0xF100 /*SC_KEYMENU*/
          //for Win-10
          IF GetFocus() != PdfView.Files.HANDLE
            nHFocusMenu := GetFocus()
            PdfView.Files.SETFOCUS
            PostMessage(nHWnd, nMsg, nWParam, nLParam)
            RETURN 1
          ENDIF

          IF GetMenu(nHWnd) == 0
            SetMenu(nHWnd, nHMenuMain)
          ELSEIF ! lMenuBar
            SetMenu(nHWnd, 0)
            RETURN 1
          ENDIF

          PdfView.PdfTimer.ENABLED := .F.
        ENDIF
        RETURN 0

      CASE 278 /*WM_INITMENU*/
        lMenuActive := .T.
        MenuCommandsEnable()
        SetOnKey(.F.)
        RETURN 0

      CASE 530 /*WM_EXITMENULOOP*/
        lMenuActive := .F.
        SetOnKey(.T.)

        //for Win-10
        IF IsWindowEnabled(nHFocusMenu)
          SetFocus(nHFocusMenu)
          nHFocusMenu := NIL
        ENDIF

        IF ! lMenuBar
          SetMenu(nHWnd, 0)
        ENDIF

        PdfView.PdfTimer.ENABLED := .T.
        RETURN 0
    ENDSWITCH

  ELSEIF _IsWindowDefined(cPanel) .and. (nHWnd == GetFormHandle(cPanel))
    SWITCH nMsg
      CASE 36 /*WM_GETMINMAXINFO*/
        nMaxY := PdfView.CLIENTAREAHEIGHT - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
        SetMinMaxTrackSize(nLParam, 200, nMaxY, PdfView.CLIENTAREAWIDTH - 100, nMaxY)
        EXIT

      CASE 532 /*WM_SIZING*/
        aRect1 := Array(4)
        aRect2 := Array(4)

        GetWindowRect(GetFormHandle(cPanel), aRect1)
        GetRectArray(nLParam, aRect2)

        SWITCH nWParam
          CASE 1 /*WMSZ_LEFT*/
          CASE 4 /*WMSZ_TOPLEFT*/
          CASE 7 /*WMSZ_BOTTOMLEFT*/
            IF aRect1[1] != aRect2[1]
              nCol := ScreenToClientCol(PdfView.HANDLE, aRect2[1])
              PdfView.Files.WIDTH := nCol
              PdfView.Files.ColumnWIDTH(1) := nCol - 4 - If(PdfView.Files.ITEMCOUNT > SendMessage(PdfView.Files.HANDLE, 0x1028 /*LVM_GETCOUNTPERPAGE*/, 0, 0), GetVScrollBarWidth(), 0)
              SetProperty(cPanel, "WIDTH", PdfView.CLIENTAREAWIDTH - nCol + (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2)
              Sumatra_FrameAdjust(cPanel)
            ENDIF
            EXIT
          OTHERWISE
            SetRectArray(nLParam, aRect1)
        ENDSWITCH
        EXIT
    ENDSWITCH
  ENDIF

RETURN NIL


FUNCTION SetLangInterface(cNewLang)
  LOCAL aLang
  LOCAL nPage
  LOCAL cMacro
  LOCAL n

  IF ! (cLang == cNewLang)
    aLang := LangStr()

    IF ! Empty(cNewLang)
      cLang := cNewLang
      nPage := Sumatra_PageNumber(cPanel)

      IF lMenuBar
        ReleaseMainMenu("PdfView")
      ELSE
        DestroyMenu(nHMenuMain)
      ENDIF
    ENDIF

    DEFINE MAINMENU OF PdfView
      DEFINE POPUP LangStr("File")
        MENUITEM LangStr("Open")                         NAME FileOpen     ACTION FileOpen()
        MENUITEM LangStr("OpenPage")                     NAME FileOpenPage ACTION FileOpen(.T.)
        MENUITEM LangStr("Close") + e"\tCtrl+W"          NAME FileClose    ACTION FileClose()
        SEPARATOR
        MENUITEM LangStr("SaveAs") + e"\tCtrl+S"         NAME FileSaveAs   ACTION Sumatra_FileSaveAs(cPanel)
        MENUITEM LangStr("Print") + e"\tCtrl+P"          NAME FilePrint    ACTION Sumatra_FilePrint(cPanel)
        MENUITEM LangStr("Proper") + e"\tCtrl+D"         NAME FileProper   ACTION Sumatra_FileProperties(cPanel)
        SEPARATOR
        MENUITEM LangStr("Refresh") + e"\tF5"            NAME FileRefresh  ACTION FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME))
        SEPARATOR
        MENUITEM LangStr("Exit") + e"\tAlt+F4"           NAME AppExit      ACTION ThisWindow.RELEASE
      END POPUP
      DEFINE POPUP LangStr("Page")
        MENUITEM LangStr("GoTo") + e"\tCtrl+G"           NAME PageGoTo     ACTION Sumatra_PageGoTo(cPanel)
        SEPARATOR
        MENUITEM LangStr("Prev")                         NAME PagePrev     ACTION Sumatra_PageGoTo(cPanel, -1)
        MENUITEM LangStr("Next")                         NAME PageNext     ACTION Sumatra_PageGoTo(cPanel,  1)
        MENUITEM LangStr("First")                        NAME PageFirst    ACTION Sumatra_PageGoTo(cPanel, -2)
        MENUITEM LangStr("Last")                         NAME PageLast     ACTION Sumatra_PageGoTo(cPanel,  2)
      END POPUP
      DEFINE POPUP LangStr("Find")
        MENUITEM LangStr("Text") + e"\tCtrl+F"           NAME FindText     ACTION Sumatra_FindText(cPanel)
        SEPARATOR
        MENUITEM LangStr("PrevOccur") + e"\tShift+F3"    NAME FindPrev     ACTION Sumatra_FindText(cPanel, -1)
        MENUITEM LangStr("NextOccur") + e"\tF3"          NAME FindNext     ACTION Sumatra_FindText(cPanel,  1)
      END POPUP
      DEFINE POPUP LangStr("Zoom")
        MENUITEM LangStr("SizeDn") + e"\tCtrl+Minus"     NAME ZoomSizeDn   ACTION Sumatra_Zoom(cPanel, -1)
        MENUITEM LangStr("SizeUp") + e"\tCtrl+Plus"      NAME ZoomSizeUp   ACTION Sumatra_Zoom(cPanel,  1)
        MENUITEM LangStr("ZoomFactor") + e"\tCtrl+Y"     NAME ZoomFactor   ACTION Sumatra_Zoom(cPanel)
        SEPARATOR
        MENUITEM LangStr("FitPage") + e"\tCtrl+0"        NAME ZoomFitPage  ACTION SetZoom(2)
        MENUITEM LangStr("ActualSize") + e"\tCtrl+1"     NAME ZoomActual   ACTION SetZoom(3)
        MENUITEM LangStr("FitWidth") + e"\tCtrl+2"       NAME ZoomFitWidth ACTION SetZoom(4)
      END POPUP
      DEFINE POPUP LangStr("Rotate")
        MENUITEM LangStr("Left") + e"\tCtrl+Shift+Minus" NAME RotateLeft   ACTION Sumatra_Rotate(cPanel, -1)
        MENUITEM LangStr("Right") + e"\tCtrl+Shift+Plus" NAME RotateRight  ACTION Sumatra_Rotate(cPanel,  1)
        MENUITEM e"&180°\tCtrl+Shift+Num*"               NAME Rotate180    ACTION Sumatra_Rotate(cPanel)
      END POPUP
      DEFINE POPUP LangStr("View")
        MENUITEM LangStr("MenuBar") + e"\tF9"            NAME MenuBar      ACTION SetMenu(PdfView.HANDLE, If((lMenuBar := ! lMenuBar), nHMenuMain, 0))
        MENUITEM LangStr("Toolbar") + e"\tF8"            NAME Toolbar      ACTION Sumatra_Toolbar(cPanel, ! Sumatra_Toolbar(cPanel))
        DEFINE POPUP LangStr("Bookmarks") NAME Bookmarks
          MENUITEM LangStr("Show") + e"\tF12"            NAME BookShow     ACTION Sumatra_Bookmarks(cPanel, ! Sumatra_Bookmarks(cPanel))
          SEPARATOR
          MENUITEM LangStr("ExpandAll") + e"\tCtrl+F12"  NAME BookExpand   ACTION Sumatra_BookmarksExpand(cPanel, .T.)
          MENUITEM LangStr("CollapseAll") + e"\tAlt+F12" NAME BookCollapse ACTION Sumatra_BookmarksExpand(cPanel, .F.)
        END POPUP
      END POPUP
      DEFINE POPUP LangStr("Settings")
        MENUITEM LangStr("OpenAtOnce")                   NAME OpenAtOnce   ACTION (lOpenAtOnce := ! lOpenAtOnce)
        MENUITEM LangStr("EscExit")                      NAME EscExit      ACTION ((lEscExit := ! lEscExit), SetOnKey(.T.))
        MENUITEM LangStr("SumatraDir")                   NAME SumatraDir   ACTION InputSumatraDir()
        DEFINE POPUP LangStr("Language")
          FOR n := 1 TO Len(aLang)
            cMacro := "SetLangInterface('" + aLang[n][2] + "')"
            MENUITEM aLang[n][1] NAME &(aLang[n][1]) ACTION &cMacro
            IF aLang[n][2] == cLang
              SetProperty("PdfView", aLang[n][1], "CHECKED", .T.)
            ENDIF
          NEXT
        END POPUP
      END POPUP
    END MENU

    nHMenuMain := GetMenu(PdfView.HANDLE)

    IF ! lMenuBar
      SetMenu(PdfView.HANDLE, 0)
    ENDIF

    IF ! Empty(cNewLang)
      PdfView.REDRAW
      FileReopen(NIL, nPage)
      SetStatusPage(.T.)
    ENDIF
  ENDIF

RETURN NIL


FUNCTION MenuCommandsEnable()
  LOCAL lPdfSelected := (PdfView.Files.VALUE > 0) .and. (! ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)))
  LOCAL lPdfOpened   := (Sumatra_FrameHandle(cPanel) != 0)
  LOCAL lPdfLoaded   := (Sumatra_PageCount(cPanel) > 0)

  GetSumatraSettings()

  PdfView.FileOpen.ENABLED     := lPdfSelected
  PdfView.FileOpenPage.ENABLED := lPdfSelected
  PdfView.FileClose.ENABLED    := lPdfOpened
  PdfView.FileSaveAs.ENABLED   := lPdfLoaded
  PdfView.FilePrint.ENABLED    := lPdfLoaded
  PdfView.FileProper.ENABLED   := lPdfLoaded
  PdfView.PageGoto.ENABLED     := lPdfLoaded
  PdfView.PagePrev.ENABLED     := lPdfLoaded
  PdfView.PageNext.ENABLED     := lPdfLoaded
  PdfView.PageFirst.ENABLED    := lPdfLoaded
  PdfView.PageLast.ENABLED     := lPdfLoaded
  PdfView.FindText.ENABLED     := lPdfLoaded
  PdfView.FindPrev.ENABLED     := lPdfLoaded
  PdfView.FindNext.ENABLED     := lPdfLoaded
  PdfView.ZoomSizeDn.ENABLED   := lPdfLoaded
  PdfView.ZoomSizeUp.ENABLED   := lPdfLoaded
  PdfView.ZoomFactor.ENABLED   := lPdfLoaded
  PdfView.ZoomFitPage.ENABLED  := lPdfLoaded
  PdfView.ZoomActual.ENABLED   := lPdfLoaded
  PdfView.ZoomFitWidth.ENABLED := lPdfLoaded
  PdfView.Rotate180.ENABLED    := lPdfLoaded
  PdfView.RotateLeft.ENABLED   := lPdfLoaded
  PdfView.RotateRight.ENABLED  := lPdfLoaded
  PdfView.Toolbar.ENABLED      := lPdfOpened
  PdfView.Bookmarks.ENABLED    := Sumatra_BookmarksExist(cPanel)
  PdfView.BookExpand.ENABLED   := lBookmarks
  PdfView.BookCollapse.ENABLED := lBookmarks

  PdfView.ZoomFitPage.CHECKED  := (nZoom == 2)
  PdfView.ZoomActual.CHECKED   := (nZoom == 3)
  PdfView.ZoomFitWidth.CHECKED := ((nZoom != 2) .and. (nZoom != 3))
  PdfView.MenuBar.CHECKED      := lMenuBar
  PdfView.Toolbar.CHECKED      := lToolbar
  PdfView.Bookmarks.CHECKED    := lBookmarks
  PdfView.BookShow.CHECKED     := lBookmarks
  PdfView.OpenAtOnce.CHECKED   := lOpenAtOnce
  PdfView.EscExit.CHECKED      := lEscExit

RETURN NIL


FUNCTION SetOnKey(lSet)

  IF lSet
    ON KEY TAB                    OF PdfView ACTION SetFocusNextCtl(.F.)
    ON KEY SHIFT+TAB              OF PdfView ACTION SetFocusNextCtl(.T.)
    ON KEY CONTROL+TAB            OF PdfView ACTION NIL
    ON KEY CONTROL+W              OF PdfView ACTION FileClose()
    ON KEY CONTROL+S              OF PdfView ACTION Sumatra_FileSaveAs(cPanel)
    ON KEY CONTROL+P              OF PdfView ACTION Sumatra_FilePrint(cPanel)
    ON KEY CONTROL+D              OF PdfView ACTION Sumatra_FileProperties(cPanel)
    ON KEY F5                     OF PdfView ACTION FileListRefresh(PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME))
    ON KEY CONTROL+G              OF PdfView ACTION Sumatra_PageGoTo(cPanel)
    ON KEY CONTROL+F              OF PdfView ACTION Sumatra_FindText(cPanel)
    ON KEY SHIFT+F3               OF PdfView ACTION Sumatra_FindText(cPanel, -1)
    ON KEY F3                     OF PdfView ACTION Sumatra_FindText(cPanel,  1)
    ON KEY CONTROL+MINUS          OF PdfView ACTION Sumatra_Zoom(cPanel, -1)
    ON KEY CONTROL+SUBTRACT       OF PdfView ACTION Sumatra_Zoom(cPanel, -1)
    ON KEY CONTROL+PLUS           OF PdfView ACTION Sumatra_Zoom(cPanel,  1)
    ON KEY CONTROL+ADD            OF PdfView ACTION Sumatra_Zoom(cPanel,  1)
    ON KEY CONTROL+Y              OF PdfView ACTION Sumatra_Zoom(cPanel)
    ON KEY CONTROL+0              OF PdfView ACTION SetZoom(2)
    ON KEY CONTROL+NUMPAD0        OF PdfView ACTION SetZoom(2)
    ON KEY CONTROL+1              OF PdfView ACTION SetZoom(3)
    ON KEY CONTROL+NUMPAD1        OF PdfView ACTION SetZoom(3)
    ON KEY CONTROL+2              OF PdfView ACTION SetZoom(4)
    ON KEY CONTROL+NUMPAD2        OF PdfView ACTION SetZoom(4)
    ON KEY CONTROL+7              OF PdfView ACTION NIL
    ON KEY CONTROL+NUMPAD7        OF PdfView ACTION NIL
    ON KEY CONTROL+8              OF PdfView ACTION NIL
    ON KEY CONTROL+NUMPAD8        OF PdfView ACTION NIL
    ON KEY CONTROL+SHIFT+MINUS    OF PdfView ACTION Sumatra_Rotate(cPanel, -1)
    ON KEY CONTROL+SHIFT+SUBTRACT OF PdfView ACTION Sumatra_Rotate(cPanel, -1)
    ON KEY CONTROL+SHIFT+PLUS     OF PdfView ACTION Sumatra_Rotate(cPanel,  1)
    ON KEY CONTROL+SHIFT+ADD      OF PdfView ACTION Sumatra_Rotate(cPanel,  1)
    ON KEY CONTROL+SHIFT+MULTIPLY OF PdfView ACTION Sumatra_Rotate(cPanel)
    ON KEY CONTROL+SHIFT+DIVIDE   OF PdfView ACTION Sumatra_Rotate(cPanel)
    ON KEY F9                     OF PdfView ACTION SetMenu(PdfView.HANDLE, If((lMenuBar := ! lMenuBar), nHMenuMain, 0))
    ON KEY F8                     OF PdfView ACTION Sumatra_Toolbar(cPanel, ! Sumatra_Toolbar(cPanel))
    ON KEY CONTROL+F12            OF PdfView ACTION Sumatra_BookmarksExpand(cPanel, .T.)
    ON KEY ALT+F12                OF PdfView ACTION Sumatra_BookmarksExpand(cPanel, .F.)
    ON KEY ALT+RETURN             OF PdfView ACTION If(IsMaximized(PdfView.HANDLE), PdfView.RESTORE, PdfView.MAXIMIZE)

    IF lEscExit
      ON KEY ESCAPE      OF PdfView ACTION PdfView.RELEASE
    ELSE
      RELEASE KEY ESCAPE OF PdfView
    ENDIF
  ELSE
    RELEASE KEY TAB                    OF PdfView
    RELEASE KEY SHIFT+TAB              OF PdfView
    RELEASE KEY CONTROL+TAB            OF PdfView
    RELEASE KEY CONTROL+W              OF PdfView
    RELEASE KEY CONTROL+S              OF PdfView
    RELEASE KEY CONTROL+P              OF PdfView
    RELEASE KEY CONTROL+D              OF PdfView
    RELEASE KEY F5                     OF PdfView
    RELEASE KEY CONTROL+G              OF PdfView
    RELEASE KEY CONTROL+F              OF PdfView
    RELEASE KEY SHIFT+F3               OF PdfView
    RELEASE KEY F3                     OF PdfView
    RELEASE KEY CONTROL+MINUS          OF PdfView
    RELEASE KEY CONTROL+SUBTRACT       OF PdfView
    RELEASE KEY CONTROL+PLUS           OF PdfView
    RELEASE KEY CONTROL+ADD            OF PdfView
    RELEASE KEY CONTROL+Y              OF PdfView
    RELEASE KEY CONTROL+0              OF PdfView
    RELEASE KEY CONTROL+NUMPAD0        OF PdfView
    RELEASE KEY CONTROL+1              OF PdfView
    RELEASE KEY CONTROL+NUMPAD1        OF PdfView
    RELEASE KEY CONTROL+2              OF PdfView
    RELEASE KEY CONTROL+NUMPAD2        OF PdfView
    RELEASE KEY CONTROL+7              OF PdfView
    RELEASE KEY CONTROL+NUMPAD7        OF PdfView
    RELEASE KEY CONTROL+8              OF PdfView
    RELEASE KEY CONTROL+NUMPAD8        OF PdfView
    RELEASE KEY CONTROL+SHIFT+MINUS    OF PdfView
    RELEASE KEY CONTROL+SHIFT+SUBTRACT OF PdfView
    RELEASE KEY CONTROL+SHIFT+PLUS     OF PdfView
    RELEASE KEY CONTROL+SHIFT+ADD      OF PdfView
    RELEASE KEY CONTROL+SHIFT+MULTIPLY OF PdfView
    RELEASE KEY CONTROL+SHIFT+DIVIDE   OF PdfView
    RELEASE KEY F9                     OF PdfView
    RELEASE KEY F8                     OF PdfView
    RELEASE KEY CONTROL+F12            OF PdfView
    RELEASE KEY ALT+F12                OF PdfView
    RELEASE KEY ALT+RETURN             OF PdfView
    RELEASE KEY ESCAPE                 OF PdfView
  ENDIF

RETURN NIL


FUNCTION GridOnKey()

  IF ! lMenuActive
    SWITCH HMG_GetLastVirtualKeyDown()
      CASE VK_F12
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0)
          Sumatra_Bookmarks(cPanel, ! Sumatra_Bookmarks(cPanel))
          PdfView.Files.SETFOCUS
        ENDIF
        EXIT
      CASE VK_LEFT
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0) .and. (PdfView.Files.CellEx(1, F_NAME) == "<..>")
          PdfView.Files.VALUE := 1
          FileOpen()
        ENDIF
        EXIT
      CASE VK_RIGHT
        HMG_CleanLastVirtualKeyDown()

        IF (GetKeyState(VK_CONTROL) >= 0) .and. (GetKeyState(VK_SHIFT) >= 0) .and. (GetKeyState(VK_MENU) >= 0) .and. ("D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)) .and. (! (PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME) == "<..>"))
          FileOpen()
        ENDIF
        EXIT
    ENDSWITCH
  ENDIF

RETURN NIL


FUNCTION SetFocusNextCtl(lPrevious)
  LOCAL nHFocus := GetFocus()
  LOCAL nHFrame := Sumatra_FrameHandle(cPanel)
  LOCAL nHBook  := Sumatra_BookmarksHandle(cPanel)

  IF nHFrame != 0
    IF nHFocus == PdfView.Files.HANDLE
      SetFocus(If(lPrevious, nHFrame, If(Sumatra_Bookmarks(cPanel), nHBook, nHFrame)))
    ELSEIF nHFocus == nHBook
      SetFocus(If(lPrevious, PdfView.Files.HANDLE, nHFrame))
    ELSE
      SetFocus(If(lPrevious, If(Sumatra_Bookmarks(cPanel), nHBook, PdfView.Files.HANDLE), PdfView.Files.HANDLE))
    ENDIF
  ENDIF

RETURN NIL


FUNCTION ResizeControls()
  LOCAL nMainCW := PdfView.CLIENTAREAWIDTH
  LOCAL nMainCH := PdfView.CLIENTAREAHEIGHT - GetWindowHeight(PdfView.STATUSBAR.HANDLE)
  LOCAL nFilesW := PdfView.Files.WIDTH
  LOCAL nFrameW := (GetProperty(cPanel, "WIDTH") - GetProperty(cPanel, "CLIENTAREAWIDTH")) / 2

  IF (nMainCW - nFilesW) < (200 - nFrameW)
    nFilesW := nMainCW - 200 - nFrameW
    PdfView.Files.WIDTH := nFilesW
    PdfView.Files.ColumnWIDTH(1) := nFilesW - 4 - If(PdfView.Files.ITEMCOUNT > SendMessage(PdfView.Files.HANDLE, 0x1028 /*LVM_GETCOUNTPERPAGE*/, 0, 0), GetVScrollBarWidth(), 0)
  ENDIF

  PdfView.Files.HEIGHT := nMainCH
  SendMessage(PdfView.Files.HANDLE, 0x1013 /*LVM_ENSUREVISIBLE*/, PdfView.Files.VALUE - 1, 0)

  //bug in MHG: SetProperty(cPanel, "ROW", nRow), SetProperty(cPanel, "COL", nCol)
  //reported: http://www.hmgforum.com/viewtopic.php?f=20&t=5178
  SetProperty(cPanel, "COL", ClientToScreenCol(GetParent(GetFormHandle(cPanel)), nFilesW))
  SetProperty(cPanel, "WIDTH", nMainCW - nFilesW + nFrameW)
  SetProperty(cPanel, "HEIGHT", nMainCH)

  Sumatra_FrameAdjust(cPanel)

RETURN NIL


FUNCTION FileListRefresh(cFileSel)
  LOCAL nSel := 1
  LOCAL aFile
  LOCAL nLen
  LOCAL n

  EnableWindowRedraw(PdfView.Files.HANDLE, .F.)
  PdfView.Files.DELETEALLITEMS

  IF ! Empty(cFileDir)
    IF VolSerial(HB_ULeft(cFileDir, 3)) == -1
      cFileSel := HB_ULeft(cFileDir, 2)
      cFileDir := ""
    ELSE
      DO WHILE ! HB_DirExists(cFileDir)
        cFileSel := ""
        cFileDir := DirParent(cFileDir)
      ENDDO
    ENDIF
  ENDIF

  IF Empty(cFileDir)
    FOR n := 65 TO 90 //from "A" to "Z"
      IF IsDisk(Chr(n))
        PdfView.Files.AddItem({Chr(n) + ":", 0, "", "", "DK"})

        IF (Chr(n) + ":") == cFileSel
          nSel := PdfView.Files.ITEMCOUNT
        ENDIF
      ENDIF
    NEXT

  ELSE
    aFile := Directory(cFileDir + "*.*", "DHS")

    FOR n := Len(aFile) TO 1 STEP -1
      IF "D" $ aFile[n][F_ATTR]
        IF (aFile[n][F_NAME] == ".") .or. (aFile[n][F_NAME] == "..")
          HB_aDel(aFile, n, .T.)
        ENDIF
      ELSEIF HMG_StrCmp(HB_FNameExt(aFile[n][F_NAME]), ".pdf", .F.) != 0
        HB_aDel(aFile, n, .T.)
      ENDIF
    NEXT

    aSort(aFile, NIL, NIL, ;
          {|a1, a2|
            IF ("D" $ a1[F_ATTR]) .and. (! ("D" $ a2[F_ATTR]))
              RETURN .T.
            ELSEIF (! ("D" $ a1[F_ATTR])) .and. ("D" $ a2[F_ATTR])
              RETURN .F.
            ENDIF
            RETURN (HMG_StrCmp(a1[F_NAME], a2[F_NAME], .F.) < 0)
          })

    HB_aIns(aFile, 1, {"..", 0, "", "", "D"}, .T.)

    nLen := Len(aFile)

    FOR n := 1 TO nLen
      IF "D" $ aFile[n][F_ATTR]
        aFile[n][F_NAME] := "<" + aFile[n][F_NAME] + ">"
      ENDIF

      PdfView.Files.AddItem(aFile[n])

      IF aFile[n][F_NAME] == cFileSel
        nSel := n
      ENDIF
    NEXT
  ENDIF

  EnableWindowRedraw(PdfView.Files.HANDLE, .T., .T.)

  PdfView.Files.VALUE := nSel
  PdfView.Files.ColumnWIDTH(1) := PdfView.Files.WIDTH - 4 - If(PdfView.Files.ITEMCOUNT > SendMessage(PdfView.Files.HANDLE, 0x1028 /*LVM_GETCOUNTPERPAGE*/, 0, 0), GetVScrollBarWidth(), 0)
  PdfView.Files.Header(1) := If(Empty(cFileDir), "Disk", cFileDir)

RETURN NIL


FUNCTION FileOpen(lAtPage)
  LOCAL cFile := PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME)
  LOCAL nPage
  LOCAL lSumatraFocus
  LOCAL cSumatraExe

  IF Empty(cFile)
    RETURN NIL
  ENDIF

  IF "D" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)
    IF "K" $ PdfView.Files.CellEx(PdfView.Files.VALUE, F_ATTR)
      IF VolSerial(DirSepAdd(cFile)) == -1
        MsgWin(cFile + CRLF + CRLF + LangStr("NoDisk"))
      ELSE
        cFileDir := DirSepAdd(cFile)
        cFile    := ""
      ENDIF
    ELSE
      cFile := HB_USubstr(cFile, 2, HMG_Len(cFile) - 2)

      IF cFile == ".."
        IF HMG_Len(cFileDir) == 3
          cFile := DirSepDel(cFileDir)
        ELSE
          cFile := "<" + HB_FNameNameExt(DirSepDel(cFileDir)) + ">"
        ENDIF
        cFileDir := DirParent(cFileDir)
      ELSE
        cFileDir := DirSepAdd(cFileDir + cFile)
        cFile    := ""
      ENDIF
    ENDIF

    FileListRefresh(cFile)
    RETURN NIL
  ENDIF

  IF HB_IsLogical(lAtPage) .and. lAtPage
    nPage := InputPageNum()

    IF HB_IsNIL(nPage)
      RETURN NIL
    ENDIF
  ENDIF

  lSumatraFocus := (GetFocus() != PdfView.Files.HANDLE)
  cSumatraExe   := "SumatraPDF.exe"

  GetSumatraSettings()

  nPage := Sumatra_FileOpen(cPanel, cFileDir + cFile, nPage, nZoom, lBookmarks, lToolbar, cLang, If(Empty(cSumatraDir), HB_DirBase(), cSumatraDir) + cSumatraExe)

  SWITCH nPage
    CASE -1
      //MsgWin("Panel window is not defined!")
      EXIT
    CASE -2
      MsgWin(If(Empty(cSumatraDir), HB_DirBase(), DirSepAdd(TrueName(cSumatraDir))) + cSumatraExe + CRLF + CRLF + LangStr("NoFile") + CRLF + LangStr("SetPath") + cSumatraExe + ".")
      InputSumatraDir()
      EXIT
    CASE -3
      MsgWin(cFileDir + cFile + CRLF + CRLF + LangStr("NoFile") + CRLF + LangStr("ListRefresh"))
      FileListRefresh("")
      EXIT
    OTHERWISE
      SetFocus(If(lSumatraFocus, Sumatra_FrameHandle(cPanel), PdfView.Files.HANDLE))
      SetStatusFile()
  ENDSWITCH

RETURN NIL


FUNCTION FileReopen(cFile, nPage)
  LOCAL nHFocus
  LOCAL nFocus

  IF Empty(cFile)
    cFile := Sumatra_FileName(cPanel)

    IF Empty(cFile)
      RETURN NIL
    ENDIF
  ENDIF

  IF ! HB_IsNumeric(nPage)
    nPage := Sumatra_PageNumber(cPanel)
  ENDIF

  nHFocus := GetFocus()

  IF nHFocus == PdfView.Files.HANDLE
    nFocus := 1
  ELSEIF nHFocus == Sumatra_BookmarksHandle(cPanel)
    nFocus := 2
  ELSE
    nFocus := 3
  ENDIF

  GetSumatraSettings()

  nPage := Sumatra_FileOpen(cPanel, cFile, nPage, nZoom, lBookmarks, lToolbar, cLang, If(Empty(cSumatraDir), HB_DirBase(), cSumatraDir) + "SumatraPDF.exe")

  IF nPage >= 0
    SWITCH nFocus
      CASE 1
        PdfView.Files.SETFOCUS
        EXIT
      CASE 2
        SetFocus(Sumatra_BookmarksHandle(cPanel))
        EXIT
      OTHERWISE
        SetFocus(Sumatra_FrameHandle(cPanel))
    ENDSWITCH
  ELSE
    PdfView.Files.SETFOCUS
  ENDIF

  SetStatusFile()

RETURN NIL


FUNCTION FileClose()

  GetSumatraSettings()
  Sumatra_FileClose(cPanel, .T.)
  SetStatusFile()

RETURN NIL


FUNCTION SetStatusFile()
  LOCAL cFile := Sumatra_FileName(cPanel)

  IF Empty(cFile)
    PdfView.TITLE := "PdfView"
    PdfView.STATUSBAR.Item(1) := ""
  ELSE
    PdfView.TITLE := "PdfView: " + HB_fNameNameExt(cFile)
    PdfView.STATUSBAR.Item(1) := cFile
  ENDIF

RETURN NIL


FUNCTION SetStatusPage(lForce)
  STATIC nPage   := 0
  STATIC nCount  := 0
  LOCAL  nPage1  := Sumatra_PageNumber(cPanel)
  LOCAL  nCount1 := Sumatra_PageCount(cPanel)

  IF lForce .or. (nPage != nPage1) .or. (nCount != nCount1)
    nPage  := nPage1
    nCount := nCount1

    PdfView.STATUSBAR.Item(2) := If(nCount == 0, "", LangStr("Page", .T.) + ": " + HB_NtoS(nPage) + "/" + HB_NtoS(nCount))
  ENDIF

  RELEASE MEMORY

RETURN NIL


FUNCTION SetZoom(nZoomNew)

  IF Sumatra_PageCount(cPanel) > 0
    nZoom := nZoomNew
    Sumatra_Zoom(cPanel, nZoom)
  ENDIF

RETURN NIL


FUNCTION GetSumatraSettings()
  LOCAL nHToolbar

  IF Sumatra_FrameHandle(cPanel) != 0
    IF Sumatra_PageCount(cPanel) > 0
      nHToolbar := Sumatra_ToolbarHandle(cPanel)

      IF HB_BitAnd(SendMessage(nHToolbar, 1042 /*TB_GETSTATE*/, 3027 /*IDT_VIEW_FIT_PAGE*/,  0), 0x01 /*TBSTATE_CHECKED*/) != 0
        nZoom := 2
      ELSEIF HB_BitAnd(SendMessage(nHToolbar, 1042 /*TB_GETSTATE*/, 3026 /*IDT_VIEW_FIT_WIDTH*/, 0), 0x01 /*TBSTATE_CHECKED*/) != 0
        nZoom := 4
      ENDIF
    ENDIF

    IF Sumatra_BookmarksExist(cPanel)
      lBookmarks := Sumatra_Bookmarks(cPanel)
    ENDIF

    lToolbar := Sumatra_Toolbar(cPanel)
  ENDIF


RETURN NIL


FUNCTION InputPageNum()
  LOCAL nPage

  DEFINE WINDOW InputPage;
    WIDTH  230 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT  74 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr("OpenFilePage");
    MODAL;
    NOSIZE

    DEFINE LABEL PageNumber
      ROW     13
      COL     50
      WIDTH   90
      HEIGHT  13
      VALUE  LangStr("PageNum")
    END LABEL

    DEFINE TEXTBOX Number
      ROW          10
      COL         140
      WIDTH        40
      HEIGHT       21
      VALUE         1
      MAXLENGTH     5
      RIGHTALIGN  .T.
      DATATYPE    NUMERIC
      ONGOTFOCUS  PostMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
      ONENTER     ((nPage := InputPage.Number.VALUE), InputPage.RELEASE)
    END TEXTBOX

    DEFINE BUTTON Open
      ROW     41
      COL     40
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr("Open")
      ACTION  ((nPage := InputPage.Number.VALUE), InputPage.RELEASE)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW      41
      COL     120
      WIDTH    70
      HEIGHT   23
      CAPTION LangStr("Cancel")
      ACTION  InputPage.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(InputPage.Open.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, InputPage.RELEASE, NIL) }, InputPage.HANDLE, 273 /*WM_COMMAND*/)

  InputPage.CenterIn(PdfView)
  InputPage.ACTIVATE

  IF HB_IsNumeric(nPage) .and. (nPage < 1)
    nPage := 1
  ENDIF

RETURN nPage


FUNCTION InputSumatraDir()
  LOCAL cDir

  DEFINE WINDOW InputDir;
    WIDTH  250 + GetSystemMetrics(7 /*SM_CXFIXEDFRAME*/) * 2;
    HEIGHT  74 + GetSystemMetrics(4 /*SM_CYCAPTION*/) + GetSystemMetrics(8 /*SM_CYFIXEDFRAME*/) * 2;
    TITLE  LangStr("SumatraDir", .T.);
    MODAL;
    NOSIZE

    DEFINE TEXTBOX Dir
      ROW         10
      COL         10
      WIDTH       230
      HEIGHT      21
      VALUE       cSumatraDir
      MAXLENGTH   260
      DATATYPE    CHARACTER
      ONGOTFOCUS  PostMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
      ONLOSTFOCUS SendMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/,    1)
      ONENTER     ((cDir := InputDir.Dir.VALUE), InputDir.RELEASE)
    END TEXTBOX

    DEFINE BUTTON Browse
      ROW     41
      COL     10
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr("Browse")
      ACTION  BrowseForSumatraDir()
    END BUTTON

    DEFINE BUTTON OK
      ROW     41
      COL     90
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr("OK")
      ACTION  ((cDir := InputDir.Dir.VALUE), InputDir.RELEASE)
    END BUTTON

    DEFINE BUTTON Cancel
      ROW     41
      COL     170
      WIDTH   70
      HEIGHT  23
      CAPTION LangStr("Cancel")
      ACTION  InputDir.RELEASE
    END BUTTON
  END WINDOW

  SendMessage(InputDir.OK.HANDLE, 244 /*BM_SETSTYLE*/, 1 /*BS_DEFPUSHBUTTON*/, 1)
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, InputDir.RELEASE, NIL) }, InputDir.HANDLE, 273 /*WM_COMMAND*/)

  InputDir.CenterIn(PdfView)
  InputDir.ACTIVATE

  IF HB_IsString(cDir)
    cSumatraDir := DirSepAdd(cDir)
  ENDIF

RETURN NIL


FUNCTION BrowseForSumatraDir()
  LOCAL cDir := DirSepAdd(TrueName(InputDir.Dir.VALUE))

  SendMessage(InputDir.OK.HANDLE,     244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
  SendMessage(InputDir.Cancel.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)

  IF ! HB_DirExists(cDir)
    cDir := HB_DirBase()
  ENDIF

  cDir := DirSepAdd(C_GetFolder(CRLF + LangStr("SelFolder"), HB_BitOr(BIF_NEWDIALOGSTYLE, BIF_NONEWFOLDERBUTTON), NIL, NIL, cDir))

  IF ! Empty(cDir)
    InputDir.Dir.VALUE := cDir
    InputDir.Dir.SETFOCUS
    SendMessage(InputDir.Browse.HANDLE, 244 /*BM_SETSTYLE*/, 0 /*BS_PUSHBUTTON*/, 1)
  ENDIF

RETURN NIL


FUNCTION MsgWin(aMsg)
  LOCAL cMsg := ""
  LOCAL nHDC
  LOCAL nHFont
  LOCAL cLine
  LOCAL nLaW, nLaH
  LOCAL nCAW

  IF HB_IsString(aMsg)
    aMsg := HB_aTokens(aMsg, .T. /*lEOL*/)
  ENDIF

  IF IsWindowVisible(GetMainFormHandle())
    DEFINE WINDOW MsgWnd;
      MODAL;
      NOSIZE
  ELSE
    DEFINE WINDOW MsgWnd;
      CHILD;
      NOMINIMIZE;
      NOMAXIMIZE;
      NOSIZE
  ENDIF

    DEFINE LABEL MsgText
      ROW 10
    END LABEL

    DEFINE BUTTON OK
      WIDTH   80
      HEIGHT  23
      CAPTION LangStr("OK")
      ACTION  MsgWnd.RELEASE
    END BUTTON
  END WINDOW

  nHDC   := GetDC(MsgWnd.MsgText.HANDLE)
  nHFont := GetWindowFont(MsgWnd.MsgText.HANDLE)
  nLaW   := 100

  FOR EACH cLine IN aMsg
    nLaW := Max(nLaW, GetTextWidth(nHDC, cLine, nHFont))
    cMsg += cLine + CRLF
  NEXT

  nLaH := GetTextHeight(nHDC, aMsg[1], nHFont) * Len(aMsg)

  ReleaseDC(MsgWnd.HANDLE, nHDC)

  nCAW := nLaW + 20

  MsgWnd.WIDTH  := nCAW + (MsgWnd.WIDTH - MsgWnd.CLIENTAREAWIDTH)
  MsgWnd.HEIGHT := nLaH + 53 + (MsgWnd.HEIGHT - MsgWnd.CLIENTAREAHEIGHT)
  MsgWnd.TITLE  := "PdfView"

  MsgWnd.MsgText.COL     := Int((nCAW - nLaW) / 2)
  MsgWnd.MsgText.WIDTH   := nLaW
  MsgWnd.MsgText.HEIGHT  := nLaH
  MsgWnd.MsgText.CAPTION := cMsg

  MsgWnd.OK.ROW := nLaH + 20
  MsgWnd.OK.COL := Int((nCAW - 80) / 2)

  C_Center(MsgWnd.HANDLE, GetActiveWindow())
  EventCreate({ || If(LoWord(EventWPARAM()) == 2 /*IDCANCEL*/, MsgWnd.RELEASE, NIL) }, MsgWnd.HANDLE, 273 /*WM_COMMAND*/)

  MsgWnd.ACTIVATE

RETURN NIL

FUNCTION IsProgramRunning()
  LOCAL cCurProcName := HMG_Upper(GetProgramFileName())
  LOCAL nCurProcID   := GetCurrentProcessId()
  LOCAL aProcID      := EnumProcessesID()
  LOCAL nPID
  LOCAL aHWnd
  LOCAL nWinPID
  LOCAL nHMainWnd
  LOCAL n

  FOR n := 1 TO Len(aProcID)
    IF (HMG_Upper(GetProcessFullName(aProcID[n])) == cCurProcName) .and. (aProcID[n] != nCurProcID)
      nPID := aProcID[n]
      EXIT
    ENDIF
  NEXT

  IF HB_IsNumeric(nPID)
    aHWnd := EnumWindows()

    FOR n := 1 TO Len(aHWnd)
      //IF (HMG_Upper(HB_ULeft(GetClassName(aHWnd[n]), 13)) == "_HMG_PDFVIEW_")
      IF IsWindowVisible(aHWnd[n]) .and. (! Empty(GetWindowText(aHWnd[n])))
        GetWindowThreadProcessId(aHWnd[n], NIL, @nWinPID)

        IF nWinPID == nPID
          nHMainWnd := aHWnd[n]
          EXIT
        ENDIF
      ENDIF
    NEXT

    IF HB_IsNumeric(nHMainWnd)
      IF IsMinimized(nHMainWnd)
        Restore(nHMainWnd)
      ELSE
        SetForegroundWindow(nHMainWnd)
      ENDIF
    ELSE
      MsgWin(LangStr("AppRunning"))
    ENDIF
  ENDIF

RETURN HB_IsNumeric(nPID)


FUNCTION SettingsRead()
  LOCAL aLine := HB_aTokens(HB_MemoRead(HB_fNameExtSet(GetProcessFullName(), "ini")), CRLF)
  LOCAL cLine
  LOCAL nPos
  LOCAL cValue

  lPdfView_Max := .F.
  nPdfView_R   := 100
  nPdfView_C   := 100
  nPdfView_W   := 650
  nPdfView_H   := 450
  nFiles_W     := 150
  nZoom        := 0
  lMenuBar     := .T.
  lToolbar     := .F.
  lBookmarks   := .T.
  lOpenAtOnce  := .F.
  lEscExit     := .F.
  cSumatraDir  := ""
  cFileDir     := HB_CWD()
  cFileLast    := ""
  cLang        := "en"

  FOR EACH cLine IN aLine
    cLine := AllTrim(cLine)
    nPos  := HB_UAt("=", cLine)

    IF nPos > 0
      cValue := HB_USubstr(cLine, nPos + 1)

      SWITCH HB_ULeft(cLine, nPos - 1)
        CASE "PdfView_Max" ; lPdfView_Max := (cValue == "T") ; EXIT
        CASE "PdfView_R"   ; nPdfView_R   := Val(cValue)     ; EXIT
        CASE "PdfView_C"   ; nPdfView_C   := Val(cValue)     ; EXIT
        CASE "PdfView_W"   ; nPdfView_W   := Val(cValue)     ; EXIT
        CASE "PdfView_H"   ; nPdfView_H   := Val(cValue)     ; EXIT
        CASE "Files_W"     ; nFiles_W     := Val(cValue)     ; EXIT
        CASE "Zoom"        ; nZoom        := Val(cValue)     ; EXIT
        CASE "MenuBar"     ; lMenuBar     := (cValue == "T") ; EXIT
        CASE "Toolbar"     ; lToolbar     := (cValue == "T") ; EXIT
        CASE "Bookmarks"   ; lBookmarks   := (cValue == "T") ; EXIT
        CASE "OpenAtOnce"  ; lOpenAtOnce  := (cValue == "T") ; EXIT
        CASE "EscExit"     ; lEscExit     := (cValue == "T") ; EXIT
        CASE "SumatraDir"  ; cSumatraDir  := cValue          ; EXIT
        CASE "FileDir"     ; cFileDir     := cValue          ; EXIT
        CASE "FileLast"    ; cFileLast    := cValue          ; EXIT
        CASE "Lang"        ; cLang        := cValue          ; EXIT
      ENDSWITCH
    ENDIF
  NEXT

  IF nPdfView_W < 400
    nPdfView_W := 400
  ENDIF

  IF nPdfView_H < 300
    nPdfView_H := 300
  ENDIF

  IF nFiles_W < 100
    nFiles_W := 100
  ENDIF

  cSumatraDir := DirSepAdd(cSumatraDir)

  IF ! Empty(cFileDir)
    IF (VolSerial(HB_ULeft(cFileDir, 3)) == -1) .or. (! HB_DirExists(cFileDir))
      cFileDir := HB_CWD()
    ELSE
      cFileDir := DirSepAdd(cFileDir)
    ENDIF
  ENDIF

  HB_CWD(HB_DirBase())

  IF aScan(LangStr(), { |aLang| aLang[2] == cLang}) == 0
    cLang := "en"
  ENDIF

RETURN NIL


FUNCTION SettingsWrite()
  LOCAL aWndPos := GetWindowNormalPos(PdfView.HANDLE)
  LOCAL cText

  GetSumatraSettings()

  cText := "PdfView_Max=" + If(PdfView.ISMAXIMIZED, "T", "F") + CRLF + ;
           "PdfView_R="   + HB_NtoS(aWndPos[2]) + CRLF + ;
           "PdfView_C="   + HB_NtoS(aWndPos[1]) + CRLF + ;
           "PdfView_W="   + HB_NtoS(aWndPos[3] - aWndPos[1]) + CRLF + ;
           "PdfView_H="   + HB_NtoS(aWndPos[4] - aWndPos[2]) + CRLF + ;
           "Files_W="     + HB_NtoS(PdfView.Files.WIDTH) + CRLF + ;
           "Zoom="        + HB_NtoS(nZoom) + CRLF + ;
           "MenuBar="     + If(lMenuBar, "T", "F") + CRLF + ;
           "Toolbar="     + If(lToolbar, "T", "F") + CRLF + ;
           "Bookmarks="   + If(lBookmarks, "T", "F") + CRLF + ;
           "OpenAtOnce="  + If(lOpenAtOnce, "T", "F") + CRLF + ;
           "EscExit="     + If(lEscExit, "T", "F") + CRLF + ;
           "SumatraDir="  + cSumatraDir + CRLF + ;
           "FileDir="     + cFileDir + CRLF + ;
           "FileLast="    + PdfView.Files.CellEx(PdfView.Files.VALUE, F_NAME) + CRLF + ;
           "Lang="        + cLang + CRLF

  HB_MemoWrit(HB_fNameExtSet(GetProcessFullName(), "ini"), cText)

RETURN NIL


FUNCTION DirSepAdd(cDir)

  IF (! Empty(cDir)) .and. (! (HB_URight(cDir, 1) == "\"))
    cDir += "\"
  ENDIF

RETURN cDir


FUNCTION DirSepDel(cDir)

  IF HB_URight(cDir, 1) == "\"
    cDir := HB_StrShrink(cDir)
  ENDIF

RETURN cDir


FUNCTION DirParent(cDir)

  cDir := DirSepDel(cDir)

RETURN HB_ULeft(cDir, HB_UTF8RAt("\", cDir))


FUNCTION LangStr(cStr, lRemoveAmpersand)
  LOCAL cText

  IF ! HB_IsChar(cStr)
    RETURN { ;
             {"English",    "en"}, ;
             {"Polish",     "pl"}, ;
             {"Portuguese", "pt"}, ;
             {"Russian",    "ru"}, ;
             {"Spanish",    "es"}  ;
           }
  ENDIF

  cText := ""

  SWITCH cLang
    CASE "pl"
      SWITCH cStr
        CASE "File"         ; cText := "&Plik"                         ; EXIT
        CASE "Open"         ; cText := "&Otwórz"                       ; EXIT
        CASE "OpenPage"     ; cText := "Otwórz na &stronie..."         ; EXIT
        CASE "Close"        ; cText := "&Zamknij"                      ; EXIT
        CASE "SaveAs"       ; cText := "Z&apisz jako..."               ; EXIT
        CASE "Print"        ; cText := "&Drukuj..."                    ; EXIT
        CASE "Proper"       ; cText := "&Właściwości..."               ; EXIT
        CASE "Refresh"      ; cText := "Odświ&eż listę"                ; EXIT
        CASE "Exit"         ; cText := "Za&kończ"                      ; EXIT
        CASE "Page"         ; cText := "&Strona"                       ; EXIT
        CASE "GoTo"         ; cText := "Przejdź &do..."                ; EXIT
        CASE "Prev"         ; cText := "Poprz&ednia"                   ; EXIT
        CASE "Next"         ; cText := "&Następna"                     ; EXIT
        CASE "First"        ; cText := "&Pierwsza"                     ; EXIT
        CASE "Last"         ; cText := "&Ostatnia"                     ; EXIT
        CASE "Find"         ; cText := "&Znajdź"                       ; EXIT
        CASE "Text"         ; cText := "&Tekst,,,"                     ; EXIT
        CASE "PrevOccur"    ; cText := "&Poprzednie wystąpienie"       ; EXIT
        CASE "NextOccur"    ; cText := "&Następne wystąpienie"         ; EXIT
        CASE "Zoom"         ; cText := "&Rozmiar"                      ; EXIT
        CASE "SizeDn"       ; cText := "&Zmniejsz"                     ; EXIT
        CASE "SizeUp"       ; cText := "&Powiększ"                     ; EXIT
        CASE "ZoomFactor"   ; cText := "&Współczynnik powiękzenia..."  ; EXIT
        CASE "FitPage"      ; cText := "&Dopasuj do strony"            ; EXIT
        CASE "ActualSize"   ; cText := "&Rozmiar rzeczywisty"          ; EXIT
        CASE "FitWidth"     ; cText := "Dopasuj do &szerokości"        ; EXIT
        CASE "Rotate"       ; cText := "&Obrót"                        ; EXIT
        CASE "Left"         ; cText := "W &lewo"                       ; EXIT
        CASE "Right"        ; cText := "W &prawo"                      ; EXIT
        CASE "View"         ; cText := "&Widok"                        ; EXIT
        CASE "MenuBar"      ; cText := "Pasek &menu"                   ; EXIT
        CASE "Toolbar"      ; cText := "Pasek &narzędzi"               ; EXIT
        CASE "Bookmarks"    ; cText := "&Zakładki PDF"                 ; EXIT
        CASE "Show"         ; cText := "&Pokaż"                        ; EXIT
        CASE "ExpandAll"    ; cText := "&Rozwiń wszystkie"             ; EXIT
        CASE "CollapseAll"  ; cText := "&Zwiń wszystkie"               ; EXIT
        CASE "Settings"     ; cText := "&Ustawienia"                   ; EXIT
        CASE "OpenAtOnce"   ; cText := "&Otwieraj PDF od razu"         ; EXIT
        CASE "EscExit"      ; cText := "&Esc - kończy program"         ; EXIT
        CASE "SumatraDir"   ; cText := "&Katalog SumatraPDF"           ; EXIT
        CASE "Language"     ; cText := "&Język"                        ; EXIT

        CASE "AppRunning"   ; cText := "Program już jest uruchomiony!" ; EXIT
        CASE "NoDisk"       ; cText := "Dysk jest niedostępny!"        ; EXIT
        CASE "NoFile"       ; cText := "Plik nie istnieje!"            ; EXIT
        CASE "SetPath"      ; cText := "Ustaw ścieżkę do: "            ; EXIT
        CASE "ListRefresh"  ; cText := "Lista zostanie odświeżona."    ; EXIT
        CASE "OpenFilePage" ; cText := "Otwórz plik na stronie"        ; EXIT
        CASE "PageNum"      ; cText := "Numer strony:"                 ; EXIT
        CASE "SelFolder"    ; cText := "Wybierz folder SumatraPDF:"    ; EXIT

        CASE "Browse"       ; cText := "&Przeglądaj..."                ; EXIT
        CASE "OK"           ; cText := "OK"                            ; EXIT
        CASE "Cancel"       ; cText := "&Anuluj"                       ; EXIT
      ENDSWITCH
      EXIT

    //Portuguese - translated by Pablo César
    CASE "pt"
      SWITCH cStr
        CASE "File"         ; cText := "Arquivo"                          ; EXIT
        CASE "Open"         ; cText := "Abrir"                            ; EXIT
        CASE "OpenPage"     ; cText := "Abrir arquivo na página"          ; EXIT
        CASE "Close"        ; cText := "Fechar"                           ; EXIT
        CASE "SaveAs"       ; cText := "Salvar como"                      ; EXIT
        CASE "Print"        ; cText := "Imprimir"                         ; EXIT
        CASE "Proper"       ; cText := "Propriedades"                     ; EXIT
        CASE "Refresh"      ; cText := "Atualizar"                        ; EXIT
        CASE "Exit"         ; cText := "Sair"                             ; EXIT
        CASE "Page"         ; cText := "Página"                           ; EXIT
        CASE "GoTo"         ; cText := "Ir para"                          ; EXIT
        CASE "Prev"         ; cText := "Anterior"                         ; EXIT
        CASE "Next"         ; cText := "Próximo"                          ; EXIT
        CASE "First"        ; cText := "Primeiro"                         ; EXIT
        CASE "Last"         ; cText := "Ultimo"                           ; EXIT
        CASE "Find"         ; cText := "Procurar"                         ; EXIT
        CASE "Text"         ; cText := "Texto"                            ; EXIT
        CASE "PrevOccur"    ; cText := "Ocorrência anterior"              ; EXIT
        CASE "NextOccur"    ; cText := "Próxima ocorrência"               ; EXIT
        CASE "Zoom"         ; cText := "Visualização"                     ; EXIT
        CASE "SizeDn"       ; cText := "Diminuir"                         ; EXIT
        CASE "SizeUp"       ; cText := "Aumentar"                         ; EXIT
        CASE "ZoomFactor"   ; cText := "Nível de ampliação"               ; EXIT
        CASE "FitPage"      ; cText := "Ajuste de página"                 ; EXIT
        CASE "ActualSize"   ; cText := "Zoom automático"                  ; EXIT
        CASE "FitWidth"     ; cText := "Ajuste de largura"                ; EXIT
        CASE "Rotate"       ; cText := "Girar"                            ; EXIT
        CASE "Left"         ; cText := "Para a esquerda"                  ; EXIT
        CASE "Right"        ; cText := "Para a direita"                   ; EXIT
        CASE "View"         ; cText := "Exibir"                           ; EXIT
        CASE "MenuBar"      ; cText := "Barra de menú"                    ; EXIT
        CASE "Toolbar"      ; cText := "Barra de ferramentas"             ; EXIT
        CASE "Bookmarks"    ; cText := "Marcadores"                       ; EXIT
        CASE "Show"         ; cText := "Exibir"                           ; EXIT
        CASE "ExpandAll"    ; cText := "Expandir todos"                   ; EXIT
        CASE "CollapseAll"  ; cText := "Recolher todos"                   ; EXIT
        CASE "Settings"     ; cText := "Configurações"                    ; EXIT
        CASE "OpenAtOnce"   ; cText := "Abrir ao click"                   ; EXIT
        CASE "EscExit"      ; cText := "Esc - Fechar o programa"          ; EXIT
        CASE "SumatraDir"   ; cText := "Pasta do SumatraPDF"              ; EXIT
        CASE "Language"     ; cText := "Idiomas"                          ; EXIT
                                                                          
        CASE "AppRunning"   ; cText := "O programa já está em execução!"  ; EXIT
        CASE "NoDisk"       ; cText := "O disco está inacessível!"        ; EXIT
        CASE "NoFile"       ; cText := "O arquivo não existe!"            ; EXIT
        CASE "SetPath"      ; cText := "Defina o caminho para: "          ; EXIT
        CASE "ListRefresh"  ; cText := "A lista será atualizada."         ; EXIT
        CASE "OpenFilePage" ; cText := "Abrir arquivo na página"          ; EXIT
        CASE "PageNum"      ; cText := "Número de página:"                ; EXIT
        CASE "SelFolder"    ; cText := "Selecione a pasta do SumatraPDF:" ; EXIT
                                                                         
        CASE "Browse"       ; cText := "Localizar..."                     ; EXIT
        CASE "OK"           ; cText := "Ok"                               ; EXIT
        CASE "Cancel"       ; cText := "Cancelar"                         ; EXIT
      ENDSWITCH
      EXIT

    CASE "ru"
      SWITCH cStr
        CASE "File"         ; cText := "файл"                       ; EXIT
        CASE "Open"         ; cText := "Открыть"                    ; EXIT
        CASE "OpenPage"     ; cText := "Открыть на странице..."     ; EXIT
        CASE "Close"        ; cText := "Закрыть"                    ; EXIT
        CASE "SaveAs"       ; cText := "Сохранить как..."           ; EXIT
        CASE "Print"        ; cText := "Печать..."                  ; EXIT
        CASE "Proper"       ; cText := "Свойства..."                ; EXIT
        CASE "Refresh"      ; cText := "Обновить список"            ; EXIT
        CASE "Exit"         ; cText := "Выход"                      ; EXIT
        CASE "Page"         ; cText := "Страница"                   ; EXIT
        CASE "GoTo"         ; cText := "Перейти к..."               ; EXIT
        CASE "Prev"         ; cText := "Предыдущая"                 ; EXIT
        CASE "Next"         ; cText := "Следущая"                   ; EXIT
        CASE "First"        ; cText := "Первая"                     ; EXIT
        CASE "Last"         ; cText := "Последняя"                  ; EXIT
        CASE "Find"         ; cText := "Найти"                      ; EXIT
        CASE "Text"         ; cText := "Текст..."                   ; EXIT
        CASE "PrevOccur"    ; cText := "Предыдущее вхождение"       ; EXIT
        CASE "NextOccur"    ; cText := "Следующее вхождение"        ; EXIT
        CASE "Zoom"         ; cText := "Масштаб"                    ; EXIT
        CASE "SizeDn"       ; cText := "Увеличить"                  ; EXIT
        CASE "SizeUp"       ; cText := "Уменьшить"                  ; EXIT
        CASE "ZoomFactor"   ; cText := "Указать масштаб..."         ; EXIT
        CASE "FitPage"      ; cText := "По размеру страницы"        ; EXIT
        CASE "ActualSize"   ; cText := "Настоящий размер"           ; EXIT
        CASE "FitWidth"     ; cText := "По ширине"                  ; EXIT
        CASE "Rotate"       ; cText := "Вращать"                    ; EXIT
        CASE "Left"         ; cText := "В лево"                     ; EXIT
        CASE "Right"        ; cText := "В право"                    ; EXIT
        CASE "View"         ; cText := "Вид"                        ; EXIT
        CASE "MenuBar"      ; cText := "Панель меню"                ; EXIT
        CASE "Toolbar"      ; cText := "Панель инструментов"        ; EXIT
        CASE "Bookmarks"    ; cText := "Закладки PDF"               ; EXIT
        CASE "Show"         ; cText := "Показать"                   ; EXIT
        CASE "ExpandAll"    ; cText := "Расширить все"              ; EXIT
        CASE "CollapseAll"  ; cText := "Свернуть все"               ; EXIT
        CASE "Settings"     ; cText := "Настройки"                  ; EXIT
        CASE "OpenAtOnce"   ; cText := "Открыть PDF немедленно"     ; EXIT
        CASE "EscExit"      ; cText := "Esc - выход"                ; EXIT
        CASE "SumatraDir"   ; cText := "Каталог SumatraPDF"         ; EXIT
        CASE "Language"     ; cText := "Язык"                       ; EXIT

        CASE "AppRunning"   ; cText := "Программа уже работает!"    ; EXIT
        CASE "NoDisk"       ; cText := "Диск не доступен!"          ; EXIT
        CASE "NoFile"       ; cText := "Файл не существует!"        ; EXIT
        CASE "SetPath"      ; cText := "Установить путь к: "        ; EXIT
        CASE "ListRefresh"  ; cText := "Список будет обновлен."     ; EXIT
        CASE "OpenFilePage" ; cText := "Открыть файл на странице"   ; EXIT
        CASE "PageNum"      ; cText := "Номер страницы:"            ; EXIT
        CASE "SelFolder"    ; cText := "Выберите папку SumatraPDF:" ; EXIT

        CASE "Browse"       ; cText := "Выбрать..."                 ; EXIT
        CASE "OK"           ; cText := "OK"                         ; EXIT
        CASE "Cancel"       ; cText := "Отмена"                     ; EXIT
      ENDSWITCH
      EXIT

    //Spanish - translated by Pablo César
    CASE "es"
      SWITCH cStr
        CASE "File"         ; cText := "Archivo"                          ; EXIT
        CASE "Open"         ; cText := "Abrir"                            ; EXIT
        CASE "OpenPage"     ; cText := "Abrir archivo en la página"       ; EXIT
        CASE "Close"        ; cText := "Cerrar"                           ; EXIT
        CASE "SaveAs"       ; cText := "Guardar como"                     ; EXIT
        CASE "Print"        ; cText := "Impresión"                        ; EXIT
        CASE "Proper"       ; cText := "Propriedades"                     ; EXIT
        CASE "Refresh"      ; cText := "Actualizar"                       ; EXIT
        CASE "Exit"         ; cText := "Salir"                            ; EXIT
        CASE "Page"         ; cText := "Página"                           ; EXIT
        CASE "GoTo"         ; cText := "Ir para"                          ; EXIT
        CASE "Prev"         ; cText := "Anterior"                         ; EXIT
        CASE "Next"         ; cText := "Próximo"                          ; EXIT
        CASE "First"        ; cText := "Primer"                           ; EXIT
        CASE "Last"         ; cText := "Ultimo"                           ; EXIT
        CASE "Find"         ; cText := "Procurar"                         ; EXIT
        CASE "Text"         ; cText := "Texto"                            ; EXIT
        CASE "PrevOccur"    ; cText := "Ocurrencia previa"                ; EXIT
        CASE "NextOccur"    ; cText := "Ocurrencia posterior"             ; EXIT
        CASE "Zoom"         ; cText := "Visualización"                    ; EXIT
        CASE "SizeDn"       ; cText := "Disminuir"                        ; EXIT
        CASE "SizeUp"       ; cText := "Aumentar"                         ; EXIT
        CASE "ZoomFactor"   ; cText := "Nivel del zoom"                   ; EXIT
        CASE "FitPage"      ; cText := "Ajuste de página"                 ; EXIT
        CASE "ActualSize"   ; cText := "Zoom automático"                  ; EXIT
        CASE "FitWidth"     ; cText := "Ajuste del ancho"                 ; EXIT
        CASE "Rotate"       ; cText := "Girar"                            ; EXIT
        CASE "Left"         ; cText := "Hacia la izquierda"               ; EXIT
        CASE "Right"        ; cText := "Hacia la derecha"                 ; EXIT
        CASE "View"         ; cText := "Exibición"                        ; EXIT
        CASE "MenuBar"      ; cText := "Barra de menú"                    ; EXIT
        CASE "Toolbar"      ; cText := "Barra de herramientas"            ; EXIT
        CASE "Bookmarks"    ; cText := "Marcadores"                       ; EXIT
        CASE "Show"         ; cText := "Exibir"                           ; EXIT
        CASE "ExpandAll"    ; cText := "Expandir todos"                   ; EXIT
        CASE "CollapseAll"  ; cText := "Retraer todos"                    ; EXIT
        CASE "Settings"     ; cText := "Configuraciones"                  ; EXIT
        CASE "OpenAtOnce"   ; cText := "Abrir de inmediato"               ; EXIT
        CASE "EscExit"      ; cText := "Esc - Cierre del programa"        ; EXIT
        CASE "SumatraDir"   ; cText := "Carpeta del SumatraPDF"           ; EXIT
        CASE "Language"     ; cText := "Idiomas"                          ; EXIT
                                                                          
        CASE "AppRunning"   ; cText := "El programa ya está en marcha!"   ; EXIT
        CASE "NoDisk"       ; cText := "El disco es inaccesible!"         ; EXIT
        CASE "NoFile"       ; cText := "El archivo no existe!"            ; EXIT
        CASE "SetPath"      ; cText := "Fije la ruta"                     ; EXIT
        CASE "ListRefresh"  ; cText := "Se actualizará la lista."         ; EXIT
        CASE "OpenFilePage" ; cText := "Abrir archivo en la página"       ; EXIT
        CASE "PageNum"      ; cText := "Número de página"                 ; EXIT
        CASE "SelFolder"    ; cText := "Seleccione la carpeta SumatraPDF" ; EXIT
                                                                         
        CASE "Browse"       ; cText := "Buscar ..."                       ; EXIT
        CASE "OK"           ; cText := "Ok"                               ; EXIT
        CASE "Cancel"       ; cText := "Cancelar"                         ; EXIT
      ENDSWITCH
      EXIT
  ENDSWITCH

  IF Empty(cText)
    SWITCH cStr
      CASE "File"         ; cText := "&File"                       ; EXIT
      CASE "Open"         ; cText := "&Open"                       ; EXIT
      CASE "OpenPage"     ; cText := "Open &at page..."            ; EXIT
      CASE "Close"        ; cText := "&Close"                      ; EXIT
      CASE "SaveAs"       ; cText := "&Save as..."                 ; EXIT
      CASE "Print"        ; cText := "&Print..."                   ; EXIT
      CASE "Proper"       ; cText := "Prop&erties..."              ; EXIT
      CASE "Refresh"      ; cText := "&Refresh list"               ; EXIT
      CASE "Exit"         ; cText := "E&xit"                       ; EXIT
      CASE "Page"         ; cText := "&Page"                       ; EXIT
      CASE "GoTo"         ; cText := "&Go to..."                   ; EXIT
      CASE "Prev"         ; cText := "&Previous"                   ; EXIT
      CASE "Next"         ; cText := "&Next"                       ; EXIT
      CASE "First"        ; cText := "&First"                      ; EXIT
      CASE "Last"         ; cText := "&Last"                       ; EXIT
      CASE "Find"         ; cText := "Fi&nd"                       ; EXIT
      CASE "Text"         ; cText := "&Text..."                    ; EXIT
      CASE "PrevOccur"    ; cText := "&Previous occurence"         ; EXIT
      CASE "NextOccur"    ; cText := "&Next occurence"             ; EXIT
      CASE "Zoom"         ; cText := "&Zoom"                       ; EXIT
      CASE "SizeDn"       ; cText := "Size &down"                  ; EXIT
      CASE "SizeUp"       ; cText := "Size &up"                    ; EXIT
      CASE "ZoomFactor"   ; cText := "&Zoom factor..."             ; EXIT
      CASE "FitPage"      ; cText := "Fit &page"                   ; EXIT
      CASE "ActualSize"   ; cText := "&Actual size"                ; EXIT
      CASE "FitWidth"     ; cText := "Fit &width"                  ; EXIT
      CASE "Rotate"       ; cText := "&Rotate"                     ; EXIT
      CASE "Left"         ; cText := "&Left"                       ; EXIT
      CASE "Right"        ; cText := "&Right"                      ; EXIT
      CASE "View"         ; cText := "&View"                       ; EXIT
      CASE "MenuBar"      ; cText := "&Menu bar"                   ; EXIT
      CASE "Toolbar"      ; cText := "&Toolbar"                    ; EXIT
      CASE "Bookmarks"    ; cText := "PDF &bookmarks"              ; EXIT
      CASE "Show"         ; cText := "&Show"                       ; EXIT
      CASE "ExpandAll"    ; cText := "&Expand all"                 ; EXIT
      CASE "CollapseAll"  ; cText := "&Collapse all"               ; EXIT
      CASE "Settings"     ; cText := "&Settings"                   ; EXIT
      CASE "OpenAtOnce"   ; cText := "&Open PDF immediately"       ; EXIT
      CASE "EscExit"      ; cText := "&Esc - exit program"         ; EXIT
      CASE "SumatraDir"   ; cText := "&SumatraPDF directory"       ; EXIT
      CASE "Language"     ; cText := "&Language"                   ; EXIT

      CASE "AppRunning"   ; cText := "Program already is running!" ; EXIT
      CASE "NoDisk"       ; cText := "Disk is not available!"      ; EXIT
      CASE "NoFile"       ; cText := "File does not exist!"        ; EXIT
      CASE "SetPath"      ; cText := "Set path to: "               ; EXIT
      CASE "ListRefresh"  ; cText := "List will be refreshed."     ; EXIT
      CASE "OpenFilePage" ; cText := "Open file at page"           ; EXIT
      CASE "PageNum"      ; cText := "Page number:"                ; EXIT
      CASE "SelFolder"    ; cText := "Select SumatraPDF folder:"   ; EXIT

      CASE "Browse"       ; cText := "&Browse..."                  ; EXIT
      CASE "OK"           ; cText := "OK"                          ; EXIT
      CASE "Cancel"       ; cText := "&Cancel"                     ; EXIT
    ENDSWITCH
  ENDIF

  IF lRemoveAmpersand == .T.
    cText := StrTran(cText, "&", "", 1, 1)
  ENDIF

RETURN cText
